devtools::install_github("dnzmarcio/ewoc", ref = "crm")
library(ewoc)

design <- "ewoc"
true_model <- "psn"
sample_size <- 40
initial_alpha <- 0.05
ncores <- 10
n_sim <- 1000

response_d1classic <- function (rho, mtd, theta, min_dose, max_dose){
  gamma <- standard_dose(dose = mtd, min_dose = min_dose, 
                         max_dose = max_dose)
  response_sim <- function(dose) {
    dose <- standard_dose(dose = dose, min_dose = min_dose, 
                          max_dose = max_dose)
    beta <- rep(NA, 2)
    beta[1] <- (gamma*sn::qsn(rho, 0, 2, 3) - min_dose*sn::qsn(theta, 0, 2, 3))/(gamma - min_dose)
    beta[2] <- (sn::qsn(theta, 0, 2, 3) - sn::qsn(rho, 0, 2, 3))/(gamma - min_dose)
    design_matrix <- cbind(1, dose)
    lp <- design_matrix %*% beta
    p <- as.numeric(sn::psn(lp, 0, 2, 3))
    out <- rbinom(n = length(dose), size = 1, prob = p)
    return(out)
  }
  out <- response_sim
  return(out)
}

dose_discrete_sim <- array(NA, dim = c(n_sim, sample_size, 5))
dose_continuous_sim <- matrix(NA, ncol = sample_size, nrow = n_sim)
dlt_discrete_sim <- array(NA, dim = c(n_sim, sample_size, 5))
dlt_continuous_sim <- matrix(NA, ncol = sample_size , nrow = n_sim)
mtd_discrete_sim <- matrix(NA, ncol = 5, nrow = n_sim)
mtd_continuous_sim <- matrix(NA, ncol = 1, nrow = n_sim)

### Logistic

## Scenario 0.2
scenario <- "0.2"

response_sim <- response_d1classic(rho = 0.15, mtd = 0.2, theta = 0.33,
                                   min_dose = 0, max_dose = 1)
dlt <- 0
dose <- 0

# Continuous

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'continuous',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1))

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_continuous_sim <- sim$dose_sim
dlt_continuous_sim <- sim$dlt_sim
mtd_continuous_sim[, 1] <- sim$mtd_sim

# Discrete 0.05

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'discrete',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            dose_set = seq(0, 1, 0.05),
                            max_increment = 0.05,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1),
                            rounding = "nearest")

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_discrete_sim[,, 1] <- sim$dose_sim
dlt_discrete_sim[,, 1] <- sim$dlt_sim
mtd_discrete_sim[, 1] <- sim$mtd_sim

# Discrete 0.10

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'discrete',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            dose_set = seq(0, 1, 0.10),
                            max_increment = 0.10,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1),
                            rounding = "nearest")

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_discrete_sim[,, 2] <- sim$dose_sim
dlt_discrete_sim[,, 2] <- sim$dlt_sim
mtd_discrete_sim[, 2] <- sim$mtd_sim

# Discrete 0.125

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.125),
                           max_increment = 0.125,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 3] <- sim$dose_sim
dlt_discrete_sim[,, 3] <- sim$dlt_sim
mtd_discrete_sim[, 3] <- sim$mtd_sim

# Discrete 0.20

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.20),
                           max_increment = 0.20,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 4] <- sim$dose_sim
dlt_discrete_sim[,, 4] <- sim$dlt_sim
mtd_discrete_sim[, 4] <- sim$mtd_sim

# Discrete 0.25

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.25),
                           max_increment = 0.25,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 5] <- sim$dose_sim
dlt_discrete_sim[,, 5] <- sim$dlt_sim
mtd_discrete_sim[, 5] <- sim$mtd_sim

save.image(paste0(design, "_", true_model, "_", 
                  sample_size, "_", scenario, ".RData"))

# Scenario 0.4
scenario <- "0.4"

dose_discrete_sim <- array(NA, dim = c(n_sim, sample_size, 5))
dose_continuous_sim <- matrix(NA, ncol = sample_size, nrow = n_sim)
dlt_discrete_sim <- array(NA, dim = c(n_sim, sample_size, 5))
dlt_continuous_sim <- matrix(NA, ncol = sample_size , nrow = n_sim)
mtd_discrete_sim <- matrix(NA, ncol = 5, nrow = n_sim)
mtd_continuous_sim <- matrix(NA, ncol = 1, nrow = n_sim)

response_sim <- response_d1classic(rho = 0.15, mtd = 0.4, theta = 0.33,
                                   min_dose = 0, max_dose = 1)

dlt <- 0
dose <- 0

# Continuous

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'continuous',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1))

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_continuous_sim <- sim$dose_sim
dlt_continuous_sim <- sim$dlt_sim
mtd_continuous_sim[, 1] <- sim$mtd_sim

# Discrete 0.05

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'discrete',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            dose_set = seq(0, 1, 0.05),
                            max_increment = 0.05,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1),
                            rounding = "nearest")

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_discrete_sim[,, 1] <- sim$dose_sim
dlt_discrete_sim[,, 1] <- sim$dlt_sim
mtd_discrete_sim[, 1] <- sim$mtd_sim

# Discrete 0.10

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'discrete',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            dose_set = seq(0, 1, 0.10),
                            max_increment = 0.10,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1),
                            rounding = "nearest")

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_discrete_sim[,, 2] <- sim$dose_sim
dlt_discrete_sim[,, 2] <- sim$dlt_sim
mtd_discrete_sim[, 2] <- sim$mtd_sim

# Discrete 0.125

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.125),
                           max_increment = 0.125,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 3] <- sim$dose_sim
dlt_discrete_sim[,, 3] <- sim$dlt_sim
mtd_discrete_sim[, 3] <- sim$mtd_sim

# Discrete 0.20

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.20),
                           max_increment = 0.20,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 4] <- sim$dose_sim
dlt_discrete_sim[,, 4] <- sim$dlt_sim
mtd_discrete_sim[, 4] <- sim$mtd_sim

# Discrete 0.25

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.25),
                           max_increment = 0.25,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 5] <- sim$dose_sim
dlt_discrete_sim[,, 5] <- sim$dlt_sim
mtd_discrete_sim[, 5] <- sim$mtd_sim

save.image(paste0(design, "_", true_model, "_", 
                  sample_size, "_", scenario, ".RData"))

# Scenario 0.6
scenario <- "0.6"

dose_discrete_sim <- array(NA, dim = c(n_sim, sample_size, 5))
dose_continuous_sim <- matrix(NA, ncol = sample_size, nrow = n_sim)
dlt_discrete_sim <- array(NA, dim = c(n_sim, sample_size, 5))
dlt_continuous_sim <- matrix(NA, ncol = sample_size , nrow = n_sim)
mtd_discrete_sim <- matrix(NA, ncol = 5, nrow = n_sim)
mtd_continuous_sim <- matrix(NA, ncol = 1, nrow = n_sim)

response_sim <- response_d1classic(rho = 0.15, mtd = 0.6, theta = 0.33,
                                   min_dose = 0, max_dose = 1)

dlt <- 0
dose <- 0

# Continuous

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'continuous',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1))

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_continuous_sim <- sim$dose_sim
dlt_continuous_sim <- sim$dlt_sim
mtd_continuous_sim[, 1] <- sim$mtd_sim

# Discrete 0.05

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'discrete',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            dose_set = seq(0, 1, 0.05),
                            max_increment = 0.05,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1),
                            rounding = "nearest")

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_discrete_sim[,, 1] <- sim$dose_sim
dlt_discrete_sim[,, 1] <- sim$dlt_sim
mtd_discrete_sim[, 1] <- sim$mtd_sim

# Discrete 0.10

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'discrete',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            dose_set = seq(0, 1, 0.10),
                            max_increment = 0.10,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1),
                            rounding = "nearest")

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_discrete_sim[,, 2] <- sim$dose_sim
dlt_discrete_sim[,, 2] <- sim$dlt_sim
mtd_discrete_sim[, 2] <- sim$mtd_sim

# Discrete 0.125

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.125),
                           max_increment = 0.125,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 3] <- sim$dose_sim
dlt_discrete_sim[,, 3] <- sim$dlt_sim
mtd_discrete_sim[, 3] <- sim$mtd_sim

# Discrete 0.20

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.20),
                           max_increment = 0.20,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 4] <- sim$dose_sim
dlt_discrete_sim[,, 4] <- sim$dlt_sim
mtd_discrete_sim[, 4] <- sim$mtd_sim

# Discrete 0.25

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.25),
                           max_increment = 0.25,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 5] <- sim$dose_sim
dlt_discrete_sim[,, 5] <- sim$dlt_sim
mtd_discrete_sim[, 5] <- sim$mtd_sim

save.image(paste0(design, "_", true_model, "_", 
                  sample_size, "_", scenario, ".RData"))

# Scenario 0.8
scenario <- "0.8"

dose_discrete_sim <- array(NA, dim = c(n_sim, sample_size, 5))
dose_continuous_sim <- matrix(NA, ncol = sample_size, nrow = n_sim)
dlt_discrete_sim <- array(NA, dim = c(n_sim, sample_size, 5))
dlt_continuous_sim <- matrix(NA, ncol = sample_size , nrow = n_sim)
mtd_discrete_sim <- matrix(NA, ncol = 5, nrow = n_sim)
mtd_continuous_sim <- matrix(NA, ncol = 1, nrow = n_sim)

response_sim <- response_d1classic(rho = 0.15, mtd = 0.8, theta = 0.33,
                                   min_dose = 0, max_dose = 1)

dlt <- 0
dose <- 0

# Continuous

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'continuous',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1))

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_continuous_sim <- sim$dose_sim
dlt_continuous_sim <- sim$dlt_sim
mtd_continuous_sim[, 1] <- sim$mtd_sim

# Discrete 0.05

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'discrete',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            dose_set = seq(0, 1, 0.05),
                            max_increment = 0.05,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1),
                            rounding = "nearest")

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_discrete_sim[,, 1] <- sim$dose_sim
dlt_discrete_sim[,, 1] <- sim$dlt_sim
mtd_discrete_sim[, 1] <- sim$mtd_sim

# Discrete 0.10

step_zero <- ewoc_d1classic(dlt ~ dose, type = 'discrete',
                            theta = 0.33, alpha = initial_alpha,
                            min_dose = 0, max_dose = 1,
                            dose_set = seq(0, 1, 0.10),
                            max_increment = 0.10,
                            rho_prior = matrix(1, ncol = 2, nrow = 1),
                            mtd_prior = matrix(1, ncol = 2, nrow = 1),
                            rounding = "nearest")

sim <- ewoc_simulation(step_zero = step_zero,
                       n_sim = n_sim, sample_size = sample_size,
                       alpha_strategy = "conditional",
                       response_sim = response_sim,
                       ncores = ncores)

dose_discrete_sim[,, 2] <- sim$dose_sim
dlt_discrete_sim[,, 2] <- sim$dlt_sim
mtd_discrete_sim[, 2] <- sim$mtd_sim

# Discrete 0.125

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.125),
                           max_increment = 0.125,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 3] <- sim$dose_sim
dlt_discrete_sim[,, 3] <- sim$dlt_sim
mtd_discrete_sim[, 3] <- sim$mtd_sim

# Discrete 0.20

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.20),
                           max_increment = 0.20,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 4] <- sim$dose_sim
dlt_discrete_sim[,, 4] <- sim$dlt_sim
mtd_discrete_sim[, 4] <- sim$mtd_sim

# Discrete 0.25

step_zero <- crm_d1classic(dlt ~ dose, type = 'discrete',
                           theta = 0.33, 
                           min_dose = 0, max_dose = 1,
                           dose_set = seq(0, 1, 0.25),
                           max_increment = 0.25,
                           rho_prior = matrix(1, ncol = 2, nrow = 1),
                           mtd_prior = matrix(1, ncol = 2, nrow = 1),
                           rounding = "nearest")

sim <- crm_simulation(step_zero = step_zero,
                      n_sim = n_sim, sample_size = sample_size,
                      response_sim = response_sim,
                      ncores = ncores)

dose_discrete_sim[,, 5] <- sim$dose_sim
dlt_discrete_sim[,, 5] <- sim$dlt_sim
mtd_discrete_sim[, 5] <- sim$mtd_sim

save.image(paste0(design, "_", true_model, "_", 
                  sample_size, "_", scenario, ".RData"))